﻿
using System;
using System.Drawing;
using System.Windows.Forms;

/*                    Interpolation by Newton                    */
/*                      www.mosismath.com                        */


namespace WindowsFormsApplication1
{

   
    public partial class MainWin : Form
    {

        public const int order = 6; // number of reference points
        public const int datapoints = 1000; // number of datapoints for the interpolation
        public const double deltaX = 0.4; // sample time 

        public TextBox[,] tbm;
        public TextBox[] tbs;
        public TextBox[] tbx;

        double[] y_k = new double[order];
        double[] x_k = new double[order];
        double[] b = new double[order];
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        
        void CalcElements(double[] x, int order, int step)
        {
            int i;
            double[] xx;
            if (order >= 1)
            {
                xx = new double[order];
                for (i = 0; i < order-1; i++)
                {
                    xx[i] = (x[i + 1] - x[i]) / (x_k[i + step] - x_k[i]);
                }
                b[step - 1] = x[0];
                CalcElements(xx, order - 1, step + 1);
            }
        }

        double Interpolate(double xp, int order)
        {
            int i, k;
            double tempYp = 0;
            double yp = 0;
            for (i = 1; i < order; i++)
            {
                tempYp = b[i];
                for (k = 0; k < i; k++)
                {
                    tempYp = tempYp * (xp - x_k[k]);
                }
                yp = yp + tempYp;
            }
            return b[0] + yp;
        }
        

        public MainWin()
        {
            int i;
            tbs = new TextBox[10];
            tbx = new TextBox[10];

            InitializeComponent();
           
            tbs[0] = textBoxY1; tbs[1] = textBoxY2; tbs[2] = textBoxY3; tbs[3] = textBoxY4; tbs[4] = textBoxY5;
            tbs[5] = textBoxY6; tbs[6] = textBoxY7; tbs[7] = textBoxY8; tbs[8] = textBoxY9; tbs[9] = textBoxY10;

            tbx[0] = textBoxX1; tbx[1] = textBoxX2; tbx[2] = textBoxX3; tbx[3] = textBoxX4; tbx[4] = textBoxX5;
            tbx[5] = textBoxX6; tbx[6] = textBoxX7; tbx[7] = textBoxX8; tbx[8] = textBoxX9; tbx[9] = textBoxX10;

            // init some sample y values
            y_k[0] = 0.5;
            y_k[1] = 11.7;
            y_k[2] = 14.8;
            y_k[3] = 4.0;
            y_k[4] = 2.2;
            y_k[5] = 0.2;

            // timestamp to y value
            for (i = 1; i <= order; i++)
            {
                x_k[i - 1] = deltaX * (double)i;
            }

            
            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    tbs[i].Text = Convert.ToString(y_k[i]);
                    tbx[i].Text = Convert.ToString(x_k[i]);
                }
                for (i = order; i < 10; i++)
                {
                    tbs[i].Visible = false;
                    tbx[i].Visible = false;
                }
            }
        }


        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(xp[j] * pGraph.Width / (double)maxTime);
                        p2.X = Convert.ToInt32(xp[j + 1] * pGraph.Width / (double)maxTime);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        if (p2.X > p1.X)
                            g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < order; j++) // mark reference points
                {
                    p1.X = Convert.ToInt32(x_k[j] * pGraph.Width / (double)maxTime);
                    if (drawCeroline)
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    else
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                }
            }
            g.Dispose();
        }

        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            int i;
            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    if (tbs[i].Text != "")
                    {
                        y_k[i] = Convert.ToDouble(tbs[i].Text);
                        x_k[i] = Convert.ToDouble(tbx[i].Text);
                    }
                    else
                    {
                        y_k[i] = 0.0;
                        x_k[i] = 0.0;
                    }
                }
            }
            CalcElements(y_k, order, 1);
            for (i = 0; i < datapoints; i++)
            {
                xp[i] = (double)(i) * x_k[order - 1] / (double)(datapoints);
                yp[i] = Interpolate(xp[i], order);
            }
           DrawGraph(datapoints, x_k[0], x_k[order - 1], true);
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, x_k[0], x_k[order - 1], false);
        }  
    }
}
